
#ifndef SEARCHRESULT
#define SEARCHRESULT 

// This is a virtual base class.  There are various
// kinds of hits with various kinds of storage requirements. 
// All derive from this base class, whose only purpose is to 
// enforce conformity to the common interface. 
// 
class SearchResultBase{
 public:

  virtual uint32 sourceStart() const =0;
  virtual uint32 sourceEnd() const =0;
  virtual uint32 queryStart() const =0;
  virtual uint32 queryEnd() const =0;

  virtual string sourceName() const = 0;
  virtual string queryName() const = 0;

  virtual uint32 sourceSize() const =0;
  virtual uint32 querySize() const = 0;
  
  //  virtual uint32  sourceIdx() const =0;
  //virtual uint32  queryIdx() const =0;

  virtual void setSourceStart(uint32 s)=0;
  virtual void setSourceEnd(uint32 s)=0;
  virtual void setQueryStart(uint32 s)=0;
  virtual void setQueryEnd(uint32 s)=0;

  virtual void setSourceName(string Name)=0;
  virtual void setQueryName(string Name)=0;

  virtual void setSourceSize(uint32 s)=0;
  virtual void setQuerySize(uint32 q)=0;

	//InitFrom(foundLocations[i],Results[oldSize]);
};

//----------------------------------------------------------------------------------------
// * SearchResult
//----------------------------------------------------------------------------------------
// SearchResult stores a single result from a hash search.  
// 
// If space for all the search results becomes an issue, we can 
// templatize this and replace source location parts of storage with 
// one of Location classes. More frequently, I suspect that it is the 
// space for the source sequence that will dominate so that this is not
// an important issue. 
//
class SearchResult: public SearchResultBase{
 public:

  SearchResult(){
    mQueryStart = 0;
    mQueryEnd = 0;
    mSourceStart = 0;
    mSourceEnd = 0;
    mSourceSize = 0;
    mQuerySize = 0;
    //    mMerHashFrequency = 0;
    //mMerGlobalFrequency = 0;
  }

  string sourceName() const {return(mSourceName);}
  string queryName() const {return(mQueryName);}
  uint32 sourceSize() const {return(mSourceSize);}
  uint32 querySize() const {return(mQuerySize);}
  
  uint32 sourceStart() const {return(mSourceStart);}
  uint32 sourceEnd() const {return(mSourceEnd);}
  uint32 queryStart() const {return(mQueryStart);}
  uint32 queryEnd() const {return(mQueryEnd);}

  //  uint32 sourceIdx() const {return(mSourceIdx);}
  //uint32 queryIdx() const {return(mQueryIdx);}

  void setSourceStart(uint32 s){mSourceStart = s;}
  void setSourceEnd(uint32 e){mSourceEnd = e;}
  void setQueryStart(uint32 s){mQueryStart = s;}
  void setQueryEnd(uint32 e){mQueryEnd = e;}
  
  void setSourceName(string Name){mSourceName = Name;}
  void setQueryName(string Name){mQueryName = Name;}

  void setSourceSize(uint32 s){mSourceSize = s;}
  void setQuerySize(uint32 q){mQuerySize = q;}

  //  void setSourceIdx(uint32 i){mSourceIdx = i;}
  //void setQueryIdx(uint32 i){mQueryIdx = i;}

  uint32  mSourceStart;
  uint32  mSourceEnd;
  
  uint32  mQueryStart;
  uint32  mQueryEnd;

  string mSourceName;
  string mQueryName;

  uint32   mSourceSize;
  uint32   mQuerySize;

  //  If we are concerned about space, we could modify this to save 
  // only an index into a global list of names.   For now, just save the
  // names. 
  // int  mSourceIdx;
  // int  mQueryIdx;

  //  unsigned int mMerHashFrequency;
  //unsigned int mMerGlobalFrequency;
};

//----------------------------------------------------------------------------------------
// * SearchResultVec
//----------------------------------------------------------------------------------------
// SearchResultVec stores a list of MerSearchResults returned from a hash search. 
//
class SearchResultVec: public vector<SearchResult> {
 public:

  // If we are concerned about the space taken up by all the names stored
  // in SearchResult, we can keep a global list of names and only store the index. 
  // However, SearchResults are typically handled in small batches, a single read
  // versus a genome or something, so that the actual space taken up is not too 
  // large at any given instance.  
  //  vector<string> mSourceNames;
  //vector<string> mQueryNames;

  void Print(ostream &fout){
    for(int i = 0;i < size();i++){
      SearchResult &R = (*this)[i];
      fout<<R.sourceName()<<"\t"<<R.sourceSize()<<"\t"<<R.sourceStart()<<"\t"<<R.sourceEnd();
      fout<<"\t"<<R.queryName()<<"\t"<<R.querySize()<<"\t"<<R.queryStart()<<"\t"<<R.queryEnd()<<endl;
    }
  }


  void PrintSSAHA(ostream &fout,bool isFwd){
    for(int i = 0;i < size();i++){
      SearchResult &R = (*this)[i];

      int rsize = R.queryEnd()-R.queryStart() + 1;      

      if (isFwd){
	fout<<"F\t";
	fout<<R.queryName()<<"\t"<<R.queryStart()<<"\t"<<R.queryEnd()<<"\t";
	fout<<R.sourceName()<<"\t"<<R.sourceStart()<<"\t"<<R.sourceEnd()<<"\t"<<rsize<<"\t100"<<endl;
      }else{
	fout<<"R\t";
	fout<<R.queryName()<<"\t"<<R.queryStart()<<"\t"<<R.queryEnd()<<"\t";
	fout<<R.sourceName()<<"\t"<<R.sourceStart()<<"\t"<<R.sourceEnd()<<"\t"<<rsize<<"\t100"<<endl;
      }
    }
  }


  void add(string &sourceName,long sourceSize,long sourceStart,long sourceEnd){
    resize(size()+1);
    SearchResult &R = (*this)[size()-1];
    R.mSourceName = sourceName;
    R.mSourceStart = sourceStart;
    R.mSourceEnd = sourceEnd;
    R.mSourceSize = sourceSize;
  }


  static bool cmpBySourceStart(const SearchResult &a, const SearchResult &b){ 
    return(a.sourceStart() < b.sourceStart());
  }
  
  static bool cmpByQueryStart(const SearchResult &a, const SearchResult &b){ 
    return(a.queryStart() < b.queryStart());
  }

  void sortByQueryStart(){
    stable_sort(this->begin(),this->end(),cmpByQueryStart);
  }

  void sortBySourceStart(){
    stable_sort(this->begin(),this->end(),cmpBySourceStart);
  }

};

#endif 
